/*
 * This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Switzerland License. To view a copy of this license, visit
 * http://creativecommons.org/licenses/by-nc-sa/2.5/ch/ or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

package ch.njol.skript.data;

import org.bukkit.block.Block;
import org.bukkit.entity.Entity;
import org.bukkit.inventory.ItemStack;

import ch.njol.skript.Skript;
import ch.njol.skript.api.Comparator;
import ch.njol.skript.util.EntityType;
import ch.njol.skript.util.ItemType;
import ch.njol.skript.util.Time;
import ch.njol.skript.util.Timeperiod;

/**
 * @author Peter Gttinger
 * 
 */
public class DefaultComparators {
	
	public DefaultComparators() {}
	
	static {
		
		// IMPORTANT!! NO Object - Object comparator
		
		// ItemStack - ItemType
		Skript.addComparator(ItemStack.class, ItemType.class, new Comparator<ItemStack, ItemType>() {
			@Override
			public Relation compare(final ItemStack is, final ItemType it) {
				return Relation.get(it.isOfType(is));
			}
		});
		
		// Block - ItemType
		Skript.addComparator(Block.class, ItemType.class, new Comparator<Block, ItemType>() {
			@Override
			public Relation compare(final Block b, final ItemType it) {
				return Relation.get(it.isOfType(b));
			}
		});
		
		// Number - Number
		Skript.addComparator(Number.class, Number.class, new Comparator<Number, Number>() {
			@Override
			public Relation compare(final Number n1, final Number n2) {
				return Relation.get(n1.doubleValue() - n2.doubleValue());
			}
		});
		
		// Entity - EntityType
		Skript.addComparator(Entity.class, EntityType.class, new Comparator<Entity, EntityType>() {
			@Override
			public ch.njol.skript.api.Comparator.Relation compare(final Entity e, final EntityType t) {
				return Relation.get(t.isInstance(e));
			}
		});
		
		// Time - Timeperiod
		Skript.addComparator(Time.class, Timeperiod.class, new Comparator<Time, Timeperiod>() {
			@Override
			public ch.njol.skript.api.Comparator.Relation compare(final Time t, final Timeperiod p) {
				return Relation.get(p.contains(t));
			}
		});
		
	}
	
}
