/*
 * This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Switzerland License. To view a copy of this license, visit
 * http://creativecommons.org/licenses/by-nc-sa/2.5/ch/ or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

package ch.njol.skript.effects;

import java.util.List;
import java.util.regex.Matcher;

import org.bukkit.Location;
import org.bukkit.event.Event;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.inventory.ItemStack;

import ch.njol.skript.Skript;
import ch.njol.skript.api.Effect;
import ch.njol.skript.api.Variable;
import ch.njol.skript.util.ItemType;
import ch.njol.util.IteratorIterable;

/**
 * 
 * @author Peter Gttinger
 * 
 */
public class EffDrop extends Effect {
	
	static {
		Skript.addEffect(EffDrop.class,
				"drop %itemtype% at %location%",
				"drop %itemtype%");
	}
	
	private Variable<ItemType> items;
	private Variable<Location> locations = null;
	
	@SuppressWarnings("unchecked")
	@Override
	public void init(final List<Variable<?>> vars, final int matchedPattern, final Matcher matcher) {
		items = (Variable<ItemType>) vars.get(0);
		if (vars.size() > 1)
			locations = (Variable<Location>) vars.get(1);
	}
	
	@Override
	public void execute(final Event e) {
		if (locations == null) {
			if (e instanceof EntityDeathEvent) {
				for (final ItemType type : items.get(e)) {
					type.addTo(((EntityDeathEvent) e).getDrops());
				}
				return;
			}
			final Location l = Skript.getEventValue(e, Location.class);
			for (final ItemType type : items.get(e)) {
				for (final ItemStack is : new IteratorIterable<ItemStack>(type.getAll()))
					l.getWorld().dropItemNaturally(l, is);
			}
			return;
		}
		for (final Location l : locations.get(e)) {
			if (l == null)
				continue;
			for (final ItemType type : items.get(e)) {
				for (final ItemStack is : new IteratorIterable<ItemStack>(type.getAll()))
					l.getWorld().dropItemNaturally(l, is);
			}
		}
	}
	
	@Override
	public String getDebugMessage(final Event e) {
		return "drop " + items.getDebugMessage(e) + (locations == null ? "" : " at " + locations.getDebugMessage(e));
	}
	
}
