/*
 * This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Switzerland License. To view a copy of this license, visit
 * http://creativecommons.org/licenses/by-nc-sa/2.5/ch/ or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

package ch.njol.skript.events;

import java.util.regex.Matcher;

import org.bukkit.Material;
import org.bukkit.entity.Enderman;
import org.bukkit.entity.Sheep;
import org.bukkit.event.Event;
import org.bukkit.event.entity.EntityChangeBlockEvent;

import ch.njol.skript.Skript;
import ch.njol.skript.api.SkriptEvent;
import ch.njol.util.Checker;

/**
 * @author Peter Gttinger
 * 
 */
public class EvtEntityBlockChange extends SkriptEvent {
	
	static {
		Skript.addEvent(EvtEntityBlockChange.class, EntityChangeBlockEvent.class, ChangeEvent.patterns);
	}
	
	private static enum ChangeEvent {
		ENDERMAN_PLACE("enderman place", new Checker<EntityChangeBlockEvent>() {
			@Override
			public boolean check(final EntityChangeBlockEvent e) {
				return e.getEntity() instanceof Enderman && e.getTo() != Material.AIR;
			}
		}),
		ENDERMAP_PICKUP("enderman pickup", new Checker<EntityChangeBlockEvent>() {
			@Override
			public boolean check(final EntityChangeBlockEvent e) {
				return e.getEntity() instanceof Enderman && e.getTo() == Material.AIR;
			}
		}),
		SHEEP_EAT("sheep eat", new Checker<EntityChangeBlockEvent>() {
			@Override
			public boolean check(final EntityChangeBlockEvent e) {
				return e.getEntity() instanceof Sheep;
			}
		});
		
		private final String pattern;
		Checker<EntityChangeBlockEvent> checker;
		
		private ChangeEvent(final String pattern, final Checker<EntityChangeBlockEvent> c) {
			this.pattern = pattern;
			checker = c;
		}
		
		static String[] patterns;
		static {
			patterns = new String[ChangeEvent.values().length];
			for (int i = 0; i < patterns.length; i++) {
				patterns[i] = values()[i].pattern;
			}
		}
	}
	
	private ChangeEvent event;
	
	@Override
	public void init(final Object[][] args, final int matchedPattern, final Matcher matcher) {
		event = ChangeEvent.values()[matchedPattern];
	}
	
	@Override
	public boolean check(final Event e) {
		if (!(e instanceof EntityChangeBlockEvent))
			return false;
		return event.checker.check((EntityChangeBlockEvent) e);
	}
	
	@Override
	public String getDebugMessage(final Event e) {
		return event.name().toLowerCase();
	}
	
}
