/*
 * This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Switzerland License. To view a copy of this license, visit
 * http://creativecommons.org/licenses/by-nc-sa/2.5/ch/ or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

package ch.njol.skript.variables;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;

import org.bukkit.event.Event;

import ch.njol.skript.Skript;
import ch.njol.skript.api.InitException;
import ch.njol.skript.api.ParseException;
import ch.njol.skript.api.Variable;
import ch.njol.skript.util.ItemData;
import ch.njol.skript.util.ItemType;

/**
 * @author Peter Gttinger
 * 
 */
public class VarIdOf extends Variable<Integer> {
	
	static {
		Skript.addVariable(VarIdOf.class, Integer.class, "ids? of %itemtype%");
	}
	
	private Variable<ItemType> types;
	
	@SuppressWarnings("unchecked")
	@Override
	public void init(final List<Variable<?>> vars, final int matchedPattern, final Matcher matcher) throws InitException, ParseException {
		types = (Variable<ItemType>) vars.get(0);
	}
	
	@Override
	public String getDebugMessage(final Event e) {
		return "id of " + types.getDebugMessage(e);
	}
	
	@Override
	protected Integer[] getAll(final Event e) {
		final ItemType[] ts = types.get(e);
		final ArrayList<Integer> r = new ArrayList<Integer>(ts.length);
		for (final ItemType t : ts) {
			for (final ItemData d : t.types) {
				r.add(Integer.valueOf(d.typeid));
			}
		}
		return r.toArray(new Integer[0]);
	}
	
	@Override
	public Class<? extends Integer> getReturnType() {
		return Integer.class;
	}
	
	@Override
	public String toString() {
		return "the id(s) of " + types;
	}
	
}
