/*
 * This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Switzerland License. To view a copy of this license, visit
 * http://creativecommons.org/licenses/by-nc-sa/2.5/ch/ or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

package ch.njol.util;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * A simple iterator to iterate over an array.
 * 
 * @author Peter Gttinger
 */
public class ArrayIterator<T> implements Iterator<T> {
	
	private final T[] array;
	private int i = -1;
	
	public ArrayIterator(final T[] array) {
		this.array = array;
	}
	
	@Override
	public boolean hasNext() {
		return ++i < array.length;
	}
	
	@Override
	public T next() {
		if (i >= array.length)
			throw new NoSuchElementException();
		return array[i];
	}
	
	/**
	 * not supported by arrays.
	 * 
	 * @throws UnsupportedOperationException always
	 */
	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}
	
}
